@@ -83,6 +83,8 @@ bool writeTest(QByteArray path)
 
 void checkComposite()
 {
+    if( qgetenv( "KDE_SKIP_ARGB_VISUALS" ) == "1" )
+        return;
     // thanks to zack rusin and frederik for pointing me in the right direction
     // for the following bits of X11 code
     dpy = XOpenDisplay(0); // open default display
@@ -115,11 +117,13 @@ void checkComposite()
                 visual = xvi[i].visual;
                 colormap = XCreateColormap(dpy, RootWindow(dpy, screen),
                                             visual, AllocNone);
-                break;
+                return;
             }
         }
 
     }
+    XCloseDisplay( dpy );
+    dpy = NULL;
 }
 
 void sanity_check( int argc, char* argv[], KAboutData* aboutDataPtr )
@@ -210,7 +214,7 @@ void sanity_check( int argc, char* argv[], KAboutData* aboutDataPtr )
     }
     if (msg.isEmpty())
     {
-        path += ".ICE-unix";
+        path += "/.ICE-unix";
         if (access(path.data(), W_OK) && (errno != ENOENT))
             msg = "No write access to '%1'.";
         else if (access(path.data(), R_OK) && (errno != ENOENT))
@@ -259,13 +263,14 @@ extern "C" KDE_EXPORT int kdemain( int argc, char* argv[] )
     checkComposite();
     KApplication *a;
 
-    if (DefaultDepth(dpy, DefaultScreen(dpy)) >= 24) { // 16bpp breaks the software logout effect for some reason???
+    if( dpy != NULL && DefaultDepth(dpy, DefaultScreen(dpy)) >= 24) // 16bpp breaks the software logout effect for some reason???
         a = new KApplication(dpy, visual ? Qt::HANDLE(visual) : 0, colormap ? Qt::HANDLE(colormap) : 0);
-    } else {
+    else
         a = new KApplication(true);
-    }
     fcntl(ConnectionNumber(QX11Info::display()), F_SETFD, 1);
 
+    a->setQuitOnLastWindowClosed(false); // #169486
+
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
     if( !QDBusConnection::sessionBus().interface()->registerService( "org.kde.ksmserver", QDBusConnectionInterface::DontQueueService ) )
