@@ -50,7 +50,7 @@ BlurEffect::BlurEffect() : Effect()
     mValid = loadData();
     if( !mValid )
         {
-        kWarning() << "Loading failed";
+        kWarning(1212) << "Loading failed";
         }
     effects->addRepaintFull();
     }
@@ -125,13 +125,13 @@ GLShader* BlurEffect::loadShader(const QString& name)
     QString vertexshader =  KGlobal::dirs()->findResource("data", "kwin/" + name + ".vert");
     if(fragmentshader.isEmpty() || vertexshader.isEmpty())
     {
-        kError() << "Couldn't locate shader files for '" << name << "'" << endl;
+        kError(1212) << "Couldn't locate shader files for '" << name << "'" << endl;
         return false;
     }
     GLShader* shader = new GLShader(vertexshader, fragmentshader);
     if(!shader->isValid())
     {
-        kError() << "Shader '" << name << "' failed to load!" << endl;
+        kError(1212) << "Shader '" << name << "' failed to load!" << endl;
         delete shader;
         return 0;
     }
@@ -184,7 +184,8 @@ void BlurEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
     //  way of knowing here whether there will be any translucent windows or
     //  not. If we'd know that there's no translucent windows then we could
     //  render straight onto screen, saving some time.
-    if( mValid /*&& mTransparentWindows*/ )
+    // HACK disable blur when there is a fullscreen effects. Needed for e.g. cube to work
+    if( mValid && !effects->activeFullScreenEffect() /*&& mTransparentWindows*/ )
         {
         // rendering everything onto render target
         effects->pushRenderTarget(mSceneTarget);
@@ -229,7 +230,7 @@ void BlurEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
                 // Restore the original matrix
                 glPopMatrix();
                 }
-//             kDebug() << "Copied" << mScreenDirty.rects().count() << "rects,    pixels:" << pixels;
+//             kDebug(1212) << "Copied" << mScreenDirty.rects().count() << "rects,    pixels:" << pixels;
             }
 
         }
@@ -241,11 +242,27 @@ void BlurEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
 
 void BlurEffect::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data )
     {
-    if( mValid /*&& mTransparentWindows*/ )
+    // HACK disable blur when there is a fullscreen effects. Needed for e.g. cube to work
+    if( mValid && !effects->activeFullScreenEffect() /*&& mTransparentWindows*/ )
         {
             if( mask & PAINT_WINDOW_TRANSLUCENT &&
                 (data.opacity != 1.0 || data.contents_opacity != 1.0 || data.decoration_opacity != 1.0 ))
             {
+
+            double blurAmount = data.opacity;
+            if( blurAmount >= 1.0 )
+            {
+                blurAmount = data.contents_opacity;
+                if( blurAmount >= 1.0 )
+                    {
+                    blurAmount = data.decoration_opacity;
+                    if( blurAmount >= 1.0 )
+                        blurAmount = 1.0;
+                    }
+            }
+            // Round to nearest 0.05
+            blurAmount = double( qRound( blurAmount * 20.0 )) / 20.0;
+
             // Make sure the blur texture is up to date
             if( mask & PAINT_SCREEN_TRANSFORMED )
                 {
@@ -258,9 +275,9 @@ void BlurEffect::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPa
             //  transformed position on the screen and thus have to update the
             //  entire screen
             if( mask & ( PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS ) )
-                updateBlurTexture( QRegion(0, 0, displayWidth(), displayHeight()) );
+                updateBlurTexture( QRegion(0, 0, displayWidth(), displayHeight()) , blurAmount );
             else
-                updateBlurTexture( mBlurDirty );
+                updateBlurTexture( mBlurDirty , blurAmount );
             mBlurDirty = QRegion();
             if( mask & PAINT_SCREEN_TRANSFORMED )
                 // Restore the original matrix
@@ -298,38 +315,38 @@ void BlurEffect::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPa
         effects->drawWindow( w, mask, region, data );
     }
 
-void BlurEffect::updateBlurTexture(const QRegion& region)
+void BlurEffect::updateBlurTexture(const QRegion& region, double blurAmount )
     {
     QRect bounding = region.boundingRect();
     QVector<QRect> rects = region.rects();
     int totalarea = 0;
-    foreach( QRect r, rects )
+    foreach( const QRect &r, rects )
         totalarea += r.width() * r.height();
     if( (int)(totalarea * 1.33 + 100 ) < bounding.width() * bounding.height() )
         {
         // Use small rects
-        updateBlurTexture(rects);
+        updateBlurTexture(rects, blurAmount);
         }
     else
         {
         // Bounding rect is probably cheaper
         QVector<QRect> tmp( 1, bounding );
-        updateBlurTexture( tmp );
+        updateBlurTexture( tmp , blurAmount );
         }
     }
 
-void BlurEffect::updateBlurTexture(const QVector<QRect>& rects)
+void BlurEffect::updateBlurTexture(const QVector<QRect>& rects, double blurAmount )
     {
     // Blur
     // First pass (vertical)
     mBlurShader->bind();
     effects->pushRenderTarget(mTmpTarget);
     mBlurShader->setAttribute("xBlur", 0.0f);
-    mBlurShader->setAttribute("yBlur", 1.0f);
+    mBlurShader->setAttribute("yBlur", float(blurAmount) );
 
     mSceneTexture->bind();
 
-    foreach( QRect r, rects )
+    foreach( const QRect &r, rects )
         {
         // We change x coordinates here because horizontal blur pass (which
         //  comes after this one) also uses pixels that are horizontally edging
@@ -349,12 +366,12 @@ void BlurEffect::updateBlurTexture(const QVector<QRect>& rects)
 
     // Second pass (horizontal)
     effects->pushRenderTarget(mBlurTarget);
-    mBlurShader->setAttribute("xBlur", 1.0f);
+    mBlurShader->setAttribute("xBlur", float(blurAmount) );
     mBlurShader->setAttribute("yBlur", 0.0f);
 
     mTmpTexture->bind();
 
-    foreach( QRect r, rects )
+    foreach( const QRect &r, rects )
         {
         glBegin(GL_QUADS);
             glVertex2f( r.x()            , r.y() + r.height() );
