@@ -23,6 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kwinconfig.h>
 #include <kwinglobals.h>
+#include "kdecoration.h"
 
 #include <QtCore/QPair>
 #include <QtCore/QRect>
@@ -56,6 +57,7 @@ class Effect;
 class WindowQuad;
 class GLRenderTarget;
 class GLShader;
+class RotationData;
 class WindowQuadList;
 class WindowPrePaintData;
 class WindowPaintData;
@@ -162,10 +164,18 @@ X-KDE-Library=kwin4_effect_cooleffect
 
 #define KWIN_EFFECT_API_MAKE_VERSION( major, minor ) (( major ) << 8 | ( minor ))
 #define KWIN_EFFECT_API_VERSION_MAJOR 0
-#define KWIN_EFFECT_API_VERSION_MINOR 21
+#define KWIN_EFFECT_API_VERSION_MINOR 59
 #define KWIN_EFFECT_API_VERSION KWIN_EFFECT_API_MAKE_VERSION( \
     KWIN_EFFECT_API_VERSION_MAJOR, KWIN_EFFECT_API_VERSION_MINOR )
 
+enum WindowQuadType
+    {
+    WindowQuadError, // for the stupid default ctor
+    WindowQuadContents,
+    WindowQuadDecoration,
+    EFFECT_QUAD_TYPE_START = 100 ///< @internal
+    };
+
 /**
  * Infinite region (i.e. a special region type saying that everything needs to be painted).
  */
@@ -272,6 +282,20 @@ class KWIN_EXPORT Effect
          **/
         virtual ~Effect();
 
+        /**
+         * Flags describing which parts of configuration have changed.
+         */
+        enum ReconfigureFlag
+            {
+            ReconfigureAll = 1 << 0 /// Everything needs to be reconfigured.
+            };
+        Q_DECLARE_FLAGS( ReconfigureFlags, ReconfigureFlag )
+
+        /**
+         * Called when configuration changes (either the effect's or KWin's global).
+         */
+        virtual void reconfigure( ReconfigureFlags flags );
+
         /**
          * Called before starting to paint the screen.
          * In this method you can:
@@ -328,6 +352,13 @@ class KWIN_EXPORT Effect
          *  do any transformations
          **/
         virtual void drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data );
+
+        /**
+         * Define new window quads so that they can be transformed by other effects.
+         * It's up to the effect to keep track of them.
+         **/
+        virtual void buildQuads( EffectWindow* w, WindowQuadList& quadList );
+
         /**
          * This function is used e.g. by the shadow effect which adds area around windows
          * that needs to be painted as well - e.g. when a window is hidden and the workspace needs
@@ -371,6 +402,22 @@ class KWIN_EXPORT Effect
         static int displayHeight();
         static QPoint cursorPos();
 
+        /**
+         * Read animation time from the configuration and possibly adjust using animationTimeFactor().
+         * The configuration value in the effect should also have special value 'default' (set using
+         * QSpinBox::setSpecialValueText()) with the value 0. This special value is adjusted
+         * using the global animation speed, otherwise the exact time configured is returned.
+         * @param cfg configuration group to read value from
+         * @param key configuration key to read value from
+         * @param defaultTime default animation time in milliseconds
+         */
+        // return type is intentionally double so that one can divide using it without losing data
+        static double animationTime( const KConfigGroup& cfg, const QString& key, int defaultTime );
+        /**
+         * @overload Use this variant if the animation time is hardcoded and not configurable
+         * in the effect itself.
+         */
+        static double animationTime( int defaultTime );
         /**
          * Linearly interpolates between @p x and @p y.
          *
@@ -448,6 +495,7 @@ class KWIN_EXPORT EffectsHandler
         virtual void paintWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data ) = 0;
         virtual void postPaintWindow( EffectWindow* w ) = 0;
         virtual void drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data ) = 0;
+        virtual void buildQuads( EffectWindow* w, WindowQuadList& quadList ) = 0;
         virtual QRect transformWindowDamage( EffectWindow* w, const QRect& r );
         // Functions for handling input - e.g. when an Expose-like effect is shown, an input window
         // covering the whole screen is created and all mouse events will be intercepted by it.
@@ -460,6 +508,10 @@ class KWIN_EXPORT EffectsHandler
         virtual bool grabKeyboard( Effect* effect ) = 0;
         virtual void ungrabKeyboard() = 0;
 
+        // Mouse polling
+        virtual void startMousePolling() = 0;
+        virtual void stopMousePolling() = 0;
+
         virtual void checkElectricBorder(const QPoint &pos, Time time) = 0;
         virtual void reserveElectricBorder( ElectricBorder border ) = 0;
         virtual void unreserveElectricBorder( ElectricBorder border ) = 0;
@@ -487,6 +539,15 @@ class KWIN_EXPORT EffectsHandler
         virtual int desktopToRight( int desktop, bool wrap ) const = 0;
         virtual int desktopUp( int desktop, bool wrap ) const = 0;
         virtual int desktopDown( int desktop, bool wrap ) const = 0;
+        /**
+         * Factor by which animation speed in the effect should be modified (multiplied).
+         * If configurable in the effect itself, the option should have also 'default'
+         * animation speed. The actual value should be determined using animationTime().
+         * Note: The factor can be also 0, so make sure your code can cope with 0ms time
+         * if used manually.
+         */
+        virtual double animationTimeFactor() const = 0;
+        virtual WindowQuadType newWindowQuadType() = 0;
 
         virtual EffectWindow* findWindow( WId id ) const = 0;
         virtual EffectWindowList stackingOrder() const = 0;
@@ -532,6 +593,22 @@ class KWIN_EXPORT EffectsHandler
         */
         virtual void registerPropertyType( long atom, bool reg ) = 0;
 
+        /**
+         * Returns @a true if the active window decoration has shadow API hooks.
+         */
+        virtual bool hasDecorationShadows() const = 0;
+        /**
+         * Returns the textures to be used in the shadow. Textures are mapped
+         * to the quad that has the same list offset. E.g. texture[2] is
+         * rendered where the third QRect that EffectWindow::shadowQuads()
+         * returns is.
+         */
+        virtual QList< QList<QImage> > shadowTextures() = 0;
+        /**
+         * Returns the texture list offset for the requested type.
+         */
+        virtual int shadowTextureList( ShadowType type ) const = 0;
+
         /**
          * Paints given text onto screen, possibly in elided form
          * @param text
@@ -542,9 +619,14 @@ class KWIN_EXPORT EffectsHandler
          **/
         bool paintText( const QString& text, const QPoint& center, int maxwidth,
                         const QColor& color, const QFont& font = QFont() );
+        bool paintText( const QString& text, const QRect& rect, const QColor& color,
+                        const QFont& font = QFont(), const Qt::Alignment& alignment = Qt::AlignCenter );
         bool paintTextWithBackground( const QString& text, const QPoint& center, int maxwidth,
                                       const QColor& color, const QColor& bgcolor,
                                       const QFont& font = QFont() );
+        bool paintTextWithBackground( const QString& text, const QRect& rect, const QColor& color,
+                                      const QColor& bgcolor, const QFont& font = QFont(),
+                                      const Qt::Alignment& alignment = Qt::AlignCenter );
 
 
         /**
@@ -567,6 +649,7 @@ class KWIN_EXPORT EffectsHandler
         int current_paint_screen;
         int current_paint_window;
         int current_draw_window;
+        int current_build_quads;
         int current_transform;
         CompositingType compositing_type;
     };
@@ -629,6 +712,7 @@ class KWIN_EXPORT EffectWindow
         virtual QSize size() const = 0;
         virtual QRect rect() const = 0;
         virtual bool isMovable() const = 0;
+        virtual bool isMovableAcrossScreens() const = 0;
         virtual bool isUserMove() const = 0;
         virtual bool isUserResize() const = 0;
         virtual QRect iconGeometry() const = 0;
@@ -739,8 +823,27 @@ class KWIN_EXPORT EffectWindow
         virtual EffectWindow* findModal() = 0;
         virtual EffectWindowList mainWindows() const = 0;
 
-        // TODO internal?
-        virtual WindowQuadList buildQuads() const = 0;
+        /**
+         * Returns the positions of the shadow quads to be rendered. All positions
+         * are relative to the window's top-left corner.
+         */
+        virtual QList<QRect> shadowQuads( ShadowType type ) const = 0;
+        /**
+         * Returns the desired opacity of the shadow.
+         */
+        virtual double shadowOpacity( ShadowType type ) const = 0;
+        /**
+         * Returns the desired brightness of the shadow.
+         */
+        virtual double shadowBrightness( ShadowType type ) const = 0;
+        /**
+         * Returns the desired saturation of the shadow.
+         */
+        virtual double shadowSaturation( ShadowType type ) const = 0;
+        /**
+         * Returns the unmodified window quad list. Can also be used to force rebuilding.
+         */
+        virtual WindowQuadList buildQuads( bool force = false ) const = 0;
     };
 
 class KWIN_EXPORT EffectWindowGroup
@@ -772,6 +875,8 @@ class KWIN_EXPORT WindowVertex
         void setY( double y );
         double originalX() const;
         double originalY() const;
+        double textureX() const;
+        double textureY() const;
         WindowVertex();
         WindowVertex( double x, double y, double tx, double ty );
     private:
@@ -782,13 +887,6 @@ class KWIN_EXPORT WindowVertex
         double tx, ty; // texture coords
     };
 
-enum WindowQuadType
-    {
-    WindowQuadError, // for the stupid default ctor
-    WindowQuadContents,
-    WindowQuadDecoration
-    };
-
 /**
  * @short Class representing one area of a window.
  *
@@ -798,11 +896,14 @@ enum WindowQuadType
 class KWIN_EXPORT WindowQuad
     {
     public:
-        explicit WindowQuad( WindowQuadType type );
+        explicit WindowQuad( WindowQuadType type, int id = -1 );
         WindowQuad makeSubQuad( double x1, double y1, double x2, double y2 ) const;
         WindowVertex& operator[]( int index );
         const WindowVertex& operator[]( int index ) const;
+        WindowQuadType type() const;
+        int id() const;
         bool decoration() const;
+        bool effect() const;
         double left() const;
         double right() const;
         double top() const;
@@ -816,7 +917,8 @@ class KWIN_EXPORT WindowQuad
     private:
         friend class WindowQuadList;
         WindowVertex verts[ 4 ];
-        WindowQuadType type; // 0 - contents, 1 - decoration
+        WindowQuadType quadType; // 0 - contents, 1 - decoration
+        int quadID;
     };
 
 class KWIN_EXPORT WindowQuadList
@@ -873,8 +975,10 @@ class KWIN_EXPORT WindowPaintData
         double decoration_opacity;
         double xScale;
         double yScale;
+        double zScale;
         int xTranslate;
         int yTranslate;
+        double zTranslate;
         /**
          * Saturation of the window, in range [0; 1]
          * 1 means that the window is unchanged, 0 means that it's completely
@@ -895,6 +999,7 @@ class KWIN_EXPORT WindowPaintData
          * Shader to be used for rendering, if any.
          */
         GLShader* shader;
+        RotationData* rotation;
     };
 
 class KWIN_EXPORT ScreenPaintData
@@ -903,8 +1008,11 @@ class KWIN_EXPORT ScreenPaintData
         ScreenPaintData();
         double xScale;
         double yScale;
+        double zScale;
         int xTranslate;
         int yTranslate;
+        double zTranslate;
+        RotationData* rotation;
     };
 
 class KWIN_EXPORT ScreenPrePaintData
@@ -914,6 +1022,23 @@ class KWIN_EXPORT ScreenPrePaintData
         QRegion paint;
     };
 
+class KWIN_EXPORT RotationData
+    {
+    public:
+        RotationData();
+        enum RotationAxis
+            {
+            XAxis,
+            YAxis,
+            ZAxis
+            };
+        RotationAxis axis;
+        float angle;
+        float xRotationPoint;
+        float yRotationPoint;
+        float zRotationPoint;
+    };
+
 /**
  * @short Helper class for restricting painting area only to allowed area.
  *
@@ -983,7 +1108,7 @@ class KWIN_EXPORT PaintClipper
 /**
  * @short Wrapper class for using timelines in KWin effects.
  *
- * This class provides an easy and specialised interface for
+ * This class provides an easy and specialized interface for
  * effects that want a non-linear timeline. Currently, most
  * it does is wrapping QTimeLine. In the future, this class
  * could help using physics animations in KWin.
@@ -1043,12 +1168,12 @@ class KWIN_EXPORT TimeLine
         };
 
         /**
-         * Creates a TimeLine and computes the progress data. The default
-         * duration can be overridden from the Effect. Usually, for larger
+         * Creates a TimeLine and computes the progress data. Usually, for larger
          * animations you want to choose values more towards 300 milliseconds.
          * For small animations, values around 150 milliseconds are sensible.
+         * Note that duration 0 is not valid.
          */
-        explicit TimeLine(const int duration = 250);
+        explicit TimeLine(int duration = 0);
 
         /**
          * Creates a copy of the TimeLine so we can have the state copied
@@ -1134,11 +1259,255 @@ class KWIN_EXPORT TimeLine
         //Q_DISABLE_COPY(TimeLine)
     };
 
+/**
+ * @internal
+ */
+template <typename T>
+class KWIN_EXPORT Motion
+    {
+    public:
+        /**
+         * Creates a new motion object. "Strength" is the amount of
+         * acceleration that is applied to the object when the target
+         * changes and "smoothness" relates to how fast the object
+         * can change its direction and speed.
+         */
+        explicit Motion( T initial, double strength, double smoothness );
+        /**
+         * Creates an exact copy of another motion object, including
+         * position, target and velocity.
+         */
+        Motion( const Motion<T> &other );
+        ~Motion();
+
+        inline T value() const { return m_value; }
+        inline void setValue( const T value ) { m_value = value; }
+        inline T target() const { return m_target; }
+        inline void setTarget( const T target ) { m_target = target; }
+        inline T velocity() const { return m_velocity; }
+        inline void setVelocity( const T velocity ) { m_velocity = velocity; }
+
+        inline double strength() const { return m_strength; }
+        inline void setStrength( const double strength ) { m_strength = strength; }
+        inline double smoothness() const { return m_smoothness; }
+        inline void setSmoothness( const double smoothness ) { m_smoothness = smoothness; }
+
+        /**
+         * The distance between the current position and the target.
+         */
+        inline T distance() const { return m_target - m_value; }
+
+        /**
+         * Calculates the new position if not at the target. Called
+         * once per frame only.
+         */
+        void calculate( const int msec );
+        /**
+         * Place the object on top of the target immediately,
+         * bypassing all movement calculation.
+         */
+        void finish();
+
+    private:
+        T m_value;
+
+        T m_target;
+        T m_velocity;
+        double m_strength;
+        double m_smoothness;
+    };
+
+/**
+ * @short A single 1D motion dynamics object.
+ *
+ * This class represents a single object that can be moved around a
+ * 1D space. Although it can be used directly by itself it is
+ * recommended to use a motion manager instead.
+ */
+class KWIN_EXPORT Motion1D : public Motion<double>
+    {
+    public:
+        explicit Motion1D( double initial = 0.0, double strength = 0.08, double smoothness = 4.0 );
+        Motion1D( const Motion1D &other );
+        ~Motion1D();
+    };
+
+/**
+ * @short A single 2D motion dynamics object.
+ *
+ * This class represents a single object that can be moved around a
+ * 2D space. Although it can be used directly by itself it is
+ * recommended to use a motion manager instead.
+ */
+class KWIN_EXPORT Motion2D : public Motion<QPointF>
+    {
+    public:
+        explicit Motion2D( QPointF initial = QPointF(), double strength = 0.08, double smoothness = 4.0 );
+        Motion2D( const Motion2D &other );
+        ~Motion2D();
+    };
+
+/**
+ * @short Helper class for motion dynamics in KWin effects.
+ *
+ * This motion manager class is intended to help KWin effect authors
+ * move windows across the screen smoothly and naturally. Once
+ * windows are registered by the manager the effect can issue move
+ * commands with the moveWindow() methods. The position of any
+ * managed window can be determined in realtime by the
+ * transformedGeometry() method. As the manager knows if any windows
+ * are moving at any given time it can also be used as a notifier as
+ * to see whether the effect is active or not.
+ */
+class KWIN_EXPORT WindowMotionManager
+    {
+    public:
+        /**
+         * Creates a new window manager object.
+         */
+        explicit WindowMotionManager( bool useGlobalAnimationModifier = true );
+        ~WindowMotionManager();
+
+        /**
+         * Register a window for managing.
+         */
+        void manage( EffectWindow *w );
+        /**
+         * Register a list of windows for managing.
+         */
+        inline void manage( EffectWindowList list )
+            {
+            for( int i = 0; i < list.size(); i++ )
+                manage( list.at( i ));
+            }
+        /**
+         * Deregister a window. All transformations applied to the
+         * window will be permanently removed and cannot be recovered.
+         */
+        void unmanage( EffectWindow *w );
+        /**
+         * Deregister all windows, returning the manager to it's
+         * originally initiated state.
+         */
+        void unmanageAll();
+        /**
+         * Determine the new positions for windows that have not
+         * reached their target. Called once per frame, usually in
+         * prePaintScreen(). Remember to set the
+         * Effect::PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS flag.
+         */
+        void calculate( int time );
+        /**
+         * Modify a registered window's paint data to make it appear
+         * at it's real location on the screen. Usually called in
+         * paintWindow(). Remember to flag the window as having been
+         * transformed in prePaintWindow() by calling
+         * WindowPrePaintData::setTransformed()
+         */
+        void apply( EffectWindow *w, WindowPaintData &data );
+        /**
+         * Set all motion targets and values back to where the
+         * windows were before transformations. The same as
+         * unmanaging then remanaging all windows.
+         */
+        void reset();
+        /**
+         * Resets the motion target and current value of a single
+         * window.
+         */
+        void reset( EffectWindow *w );
+
+        /**
+         * As the manager to move the window to the target position
+         * with the specified scale. If `yScale` is not provided or
+         * set to 0.0, `scale` will be used as the scale in the
+         * vertical direction as well as in the horizontal direction.
+         */
+        void moveWindow( EffectWindow *w, QPoint target, double scale = 1.0, double yScale = 0.0 );
+        /**
+         * This is an overloaded method, provided for convenience.
+         * 
+         * Ask the manager to move the window to the target rectangle.
+         * Automatically determines scale.
+         */
+        inline void moveWindow( EffectWindow *w, QRect target )
+            {
+            // TODO: Scale might be slightly different in the comparison due to rounding
+            moveWindow( w, target.topLeft(),
+                target.width() / double( w->width() ), target.height() / double( w->height() ));
+            }
+
+        /**
+         * Retrieve the current tranformed geometry of a registered
+         * window.
+         */
+        QRectF transformedGeometry( EffectWindow *w ) const;
+        /**
+         * Return the window that has it's transformed geometry under
+         * the specified point. It is recommended to use the stacking
+         * order as it's what the user sees, but it is slightly
+         * slower to process.
+         */
+        EffectWindow* windowAtPoint( QPoint point, bool useStackingOrder = true ) const;
+
+        /**
+         * Return a list of all currently registered windows.
+         */
+        inline EffectWindowList managedWindows() const { return m_managedWindows.keys(); }
+        /**
+         * Returns whether or not a specified window is being managed
+         * by this manager object.
+         */
+        inline bool isManaging( EffectWindow *w ) { return m_managedWindows.contains( w ); }
+        /**
+         * Returns whether or not this manager object is actually
+         * managing any windows or not.
+         */
+        inline bool managingWindows() { return !m_managedWindows.empty(); }
+        /**
+         * Returns whether all windows have reached their targets yet
+         * or not. Can be used to see if an effect should be
+         * processed and displayed or not.
+         */
+        inline bool areWindowsMoving() { return m_movingWindows > 0; }
+
+    private:
+        bool m_useGlobalAnimationModifier;
+        struct WindowMotion
+            { // TODO: Rotation, etc?
+            Motion2D translation; // Absolute position
+            Motion2D scale; // xScale and yScale
+            };
+        QHash<EffectWindow*, WindowMotion> m_managedWindows;
+        uint m_movingWindows;
+    };
+
 /**
  * Pointer to the global EffectsHandler object.
  **/
 extern KWIN_EXPORT EffectsHandler* effects;
 
+/***************************************************************
+ EffectsHandler
+***************************************************************/
+
+inline
+bool EffectsHandler::paintText( const QString& text, const QPoint& center, int maxwidth,
+        const QColor& color, const QFont& font )
+{
+    return paintText( text, QRect( center.x() - maxwidth / 2, center.y() - 5000, maxwidth, 10000 ),
+        color, font, Qt::AlignCenter );
+}
+
+inline
+bool EffectsHandler::paintTextWithBackground( const QString& text, const QPoint& center, int maxwidth,
+        const QColor& color, const QColor& bgcolor, const QFont& font )
+{
+    return paintTextWithBackground( text,
+        QRect( center.x() - maxwidth / 2, center.y() - 5000, maxwidth, 10000 ),
+        color, bgcolor, font, Qt::AlignCenter );
+}
+
 /***************************************************************
  WindowVertex
 ***************************************************************/
@@ -1179,6 +1548,18 @@ double WindowVertex::originalY() const
     return oy;
     }
 
+inline
+double WindowVertex::textureX() const
+    {
+    return tx;
+    }
+
+inline
+double WindowVertex::textureY() const
+    {
+    return ty;
+    }
+
 inline
 void WindowVertex::move( double x, double y )
     {
@@ -1203,8 +1584,9 @@ void WindowVertex::setY( double y )
 ***************************************************************/
 
 inline
-WindowQuad::WindowQuad( WindowQuadType t )
-    : type( t )
+WindowQuad::WindowQuad( WindowQuadType t, int id )
+    : quadType( t )
+    , quadID( id )
     {
     }
 
@@ -1222,11 +1604,31 @@ const WindowVertex& WindowQuad::operator[]( int index ) const
     return verts[ index ];
     }
 
+inline
+WindowQuadType WindowQuad::type() const
+    {
+    assert( quadType != WindowQuadError );
+    return quadType;
+    }
+
+inline
+int WindowQuad::id() const
+    {
+    return quadID;
+    }
+
 inline
 bool WindowQuad::decoration() const
     {
-    assert( type != WindowQuadError );
-    return type == WindowQuadDecoration;
+    assert( quadType != WindowQuadError );
+    return quadType == WindowQuadDecoration;
+    }
+
+inline
+bool WindowQuad::effect() const
+    {
+    assert( quadType != WindowQuadError );
+    return quadType >= EFFECT_QUAD_TYPE_START;
     }
 
 inline
@@ -1286,6 +1688,59 @@ double WindowQuad::originalBottom() const
     return verts[ 2 ].oy;
     }
 
+/***************************************************************
+ Motion
+***************************************************************/
+
+template <typename T>
+Motion<T>::Motion( T initial, double strength, double smoothness )
+    :   m_value( initial )
+    ,   m_target( initial )
+    ,   m_velocity()
+    ,   m_strength( strength )
+    ,   m_smoothness( smoothness )
+    {
+    }
+
+template <typename T>
+Motion<T>::Motion( const Motion &other )
+    :   m_value( other.value() )
+    ,   m_target( other.target() )
+    ,   m_velocity( other.velocity() )
+    ,   m_strength( other.strength() )
+    ,   m_smoothness( other.smoothness() )
+    {
+    }
+
+template <typename T>
+Motion<T>::~Motion()
+    {
+    }
+
+template <typename T>
+void Motion<T>::calculate( const int msec )
+    {
+    if( m_value == m_target && m_velocity == T() ) // At target and not moving
+        return;
+
+    // Poor man's time independent calculation
+    int steps = qMax( 1, msec / 5 );
+    for( int i = 0; i < steps; i++ )
+        {
+        T diff = m_target - m_value;
+        T strength = diff * m_strength;
+        m_velocity = ( m_smoothness * m_velocity + strength ) / ( m_smoothness + 1.0 );
+        m_value += m_velocity;
+        }
+    }
+
+template <typename T>
+void Motion<T>::finish()
+    {
+    m_value = m_target;
+    m_velocity = T();
+    }
+
 } // namespace
 
 /** @} */
